<?php

namespace UnicaenDbAnonym\Controller;

use Exception;
use Laminas\Mvc\Console\Controller\AbstractConsoleController;
use UnicaenApp\Exception\RuntimeException;
use UnicaenDbAnonym\Service\AnonymServiceAwareTrait;

class ConsoleController extends AbstractConsoleController
{
    use AnonymServiceAwareTrait;

    /**
     * @var string
     */
    protected $outputDir;

    public function genererAction()
    {
        $start = microtime(true);

        $this->console->writeLine("Generation des scripts SQL...");

        $anonymisationScriptPath = $this->anonymService->getAnonymisationScriptPath();
        $restaurationScriptPath = $this->anonymService->getRestaurationScriptPath();
        try {
            $gen = $this->anonymService->generer();
            foreach ($gen as ['table' => $table, 'fields' => $fields, 'count' => $count]) {
                $this->console->writeLine(
                    sprintf("- Table %s (colonnes %s) : %d lignes", $table, implode(', ', $fields), $count)
                );
            }
        } catch (Exception $e) {
            throw new RuntimeException("Une erreur est survenue pendant la génération des scripts SQL.", null, $e);
        }

        $end = microtime(true);

        $this->console->writeLine("> Script d'anonymisation : " . realpath($anonymisationScriptPath));
        $this->console->writeLine("> Script de restauration : " . realpath($restaurationScriptPath));
        $this->console->writeLine(sprintf("Duree : %.2f s", $end - $start));
    }

    public function anonymiserAction()
    {
        $scriptPath = $this->anonymService->getAnonymisationScriptPath();

        $this->console->writeLine("Lancement du script d'anonymisation '$scriptPath'...");

        $start = microtime(true);
        try {
            $result = $this->anonymService->anonymiser();
        } catch (Exception $e) {
            throw new RuntimeException("Une erreur est survenue lors du lancement du script '$scriptPath'.", null, $e);
        }
        $end = microtime(true);

        $this->console->writeLine("> Résultat : " . ($result->isSuccess() ? 'Succès' : 'Échec'));
        $this->console->writeLine("> Log file : " . $result->getLogFilePath());
        $this->console->writeLine(sprintf("Duree : %.2f s", $end - $start));
    }

    public function restaurerAction()
    {
        $scriptPath = $this->anonymService->getRestaurationScriptPath();

        $this->console->writeLine("Lancement du script de restauration '$scriptPath'...");

        $start = microtime(true);
        try {
            $result = $this->anonymService->restaurer();
        } catch (Exception $e) {
            throw new RuntimeException("Une erreur est survenue lors du lancement du script '$scriptPath'.", null, $e);
        }
        $end = microtime(true);

        $this->console->writeLine("> Résultat : " . ($result->isSuccess() ? 'Succès' : 'Échec'));
        $this->console->writeLine("> Log file : " . $result->getLogFilePath());
        $this->console->writeLine(sprintf("Duree : %.2f s", $end - $start));
    }
}